@@ -3,43 +3,35 @@
 
 #pragma once
 
-#include <qmldesignercorelib_global.h>
-
-#include <model.h>
-#include <modelnode.h>
-#include <abstractproperty.h>
-#include <documentmessage.h>
-#include <rewritertransaction.h>
+#include "model.h"
+#include "modelnode.h"
+#include "qmldesignercorelib_global.h"
 #include <commondefines.h>
 
 #include <QObject>
 #include <QPointer>
-#include <QVector3D>
-
-#include <functional>
-#include <memory>
 
 QT_BEGIN_NAMESPACE
-class QStyle;
-class QToolButton;
 class QImage;
 class QPixmap;
+class QVector3D;
 QT_END_NAMESPACE
 
-namespace QmlDesigner {
-    namespace Internal {
-    class InternalNode;
-    using InternalNodePointer = std::shared_ptr<InternalNode>;
-    }
-}
-
 namespace QmlDesigner {
 
+class AbstractProperty;
+class DocumentMessage;
+class ExternalDependenciesInterface;
 class NodeInstanceView;
-class RewriterView;
 class QmlModelState;
 class QmlTimeline;
-class ExternalDependenciesInterface;
+class RewriterTransaction;
+class RewriterView;
+
+namespace Internal {
+class InternalNode;
+using InternalNodePointer = std::shared_ptr<InternalNode>;
+}
 
 enum DesignerWidgetFlags {
     DisableOnError,
@@ -60,6 +52,7 @@ public:
 
     QString uniqueId;
     QString tabName;
+    QString feedbackDisplayName;
     QWidget *widget = nullptr;
     int placementPriority;
     PlacementHint placementHint;
@@ -74,18 +67,19 @@ public:
     Q_FLAGS(PropertyChangeFlag PropertyChangeFlags)
 
     enum PropertyChangeFlag {
-      NoAdditionalChanges = 0x0,
-      PropertiesAdded = 0x1,
+      NoAdditionalChanges    = 0x0,
+      PropertiesAdded        = 0x1,
       EmptyPropertiesRemoved = 0x2
     };
     Q_DECLARE_FLAGS(PropertyChangeFlags, PropertyChangeFlag)
+
     AbstractView(ExternalDependenciesInterface &externalDependencies)
         : m_externalDependencies{externalDependencies}
     {}
 
     ~AbstractView() override;
 
-    Model* model() const;
+    Model *model() const;
     bool isAttached() const;
 
     RewriterTransaction beginRewriterTransaction(const QByteArray &identifier);
@@ -127,7 +121,7 @@ public:
     QList<ModelNode> allModelNodes() const;
     QList<ModelNode> allModelNodesOfType(const NodeMetaInfo &typeName) const;
 
-    void emitDocumentMessage(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings = QList<DocumentMessage>());
+    void emitDocumentMessage(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings = {});
     void emitDocumentMessage(const QString &error);
     void emitCustomNotification(const QString &identifier);
     void emitCustomNotification(const QString &identifier, const QList<ModelNode> &nodeList);
@@ -155,19 +149,25 @@ public:
 
     virtual void nodeCreated(const ModelNode &createdNode);
     virtual void nodeAboutToBeRemoved(const ModelNode &removedNode);
-    virtual void nodeRemoved(const ModelNode &removedNode, const NodeAbstractProperty &parentProperty, PropertyChangeFlags propertyChange);
-    virtual void nodeAboutToBeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent, const NodeAbstractProperty &oldPropertyParent, AbstractView::PropertyChangeFlags propertyChange);
-    virtual void nodeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent, const NodeAbstractProperty &oldPropertyParent, AbstractView::PropertyChangeFlags propertyChange);
-    virtual void nodeIdChanged(const ModelNode& node, const QString& newId, const QString& oldId);
-    virtual void propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList);
-    virtual void propertiesRemoved(const QList<AbstractProperty>& propertyList);
-    virtual void variantPropertiesChanged(const QList<VariantProperty>& propertyList, PropertyChangeFlags propertyChange);
+    virtual void nodeRemoved(const ModelNode &removedNode, const NodeAbstractProperty &parentProperty,
+                             PropertyChangeFlags propertyChange);
+    virtual void nodeAboutToBeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent,
+                                         const NodeAbstractProperty &oldPropertyParent, PropertyChangeFlags propertyChange);
+    virtual void nodeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent,
+                                const NodeAbstractProperty &oldPropertyParent, PropertyChangeFlags propertyChange);
+    virtual void nodeIdChanged(const ModelNode &node, const QString &newId, const QString &oldId);
+    virtual void propertiesAboutToBeRemoved(const QList<AbstractProperty> &propertyList);
+    virtual void propertiesRemoved(const QList<AbstractProperty> &propertyList);
+    virtual void variantPropertiesChanged(const QList<VariantProperty> &propertyList,
+                                          PropertyChangeFlags propertyChange);
     virtual void bindingPropertiesAboutToBeChanged(const QList<BindingProperty> &propertyList);
-    virtual void bindingPropertiesChanged(const QList<BindingProperty>& propertyList, PropertyChangeFlags propertyChange);
-    virtual void signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty>& propertyList, PropertyChangeFlags propertyChange);
-    virtual void signalDeclarationPropertiesChanged(const QVector<SignalDeclarationProperty>& propertyList, PropertyChangeFlags propertyChange);
+    virtual void bindingPropertiesChanged(const QList<BindingProperty> &propertyList, PropertyChangeFlags propertyChange);
+    virtual void signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty> &propertyList,
+                                                PropertyChangeFlags propertyChange);
+    virtual void signalDeclarationPropertiesChanged(const QVector<SignalDeclarationProperty> &propertyList,
+                                                    PropertyChangeFlags propertyChange);
     virtual void rootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion);
-    virtual void nodeTypeChanged(const ModelNode& node, const TypeName &type, int majorVersion, int minorVersion);
+    virtual void nodeTypeChanged(const ModelNode &node, const TypeName &type, int majorVersion, int minorVersion);
 
     virtual void instancePropertyChanged(const QList<QPair<ModelNode, PropertyName> > &propertyList);
     virtual void instanceErrorChanged(const QVector<ModelNode> &errorNodeList);
@@ -195,9 +195,9 @@ public:
                                   const ModelNode &movedNode,
                                   int oldIndex);
 
-    virtual void importsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports);
-    virtual void possibleImportsChanged(const QList<Import> &possibleImports);
-    virtual void usedImportsChanged(const QList<Import> &usedImports);
+    virtual void importsChanged(const Imports &addedImports, const Imports &removedImports);
+    virtual void possibleImportsChanged(const Imports &possibleImports);
+    virtual void usedImportsChanged(const Imports &usedImports);
 
     virtual void auxiliaryDataChanged(const ModelNode &node,
                                       AuxiliaryDataKeyView type,
@@ -228,8 +228,8 @@ public:
 
     void ensureMaterialLibraryNode();
     ModelNode materialLibraryNode();
+    bool isPartOfMaterialLibrary(const ModelNode &node);
     ModelNode active3DSceneNode();
-    void assignMaterialTo3dModel(const ModelNode &modelNode, const ModelNode &materialNode = {});
     ModelNode getTextureDefaultInstance(const QString &source);
 
     const NodeInstanceView *nodeInstanceView() const;
@@ -244,7 +244,6 @@ public:
     int minorQtQuickVersion() const;
 
     void resetView();
-
     void resetPuppet();
 
     virtual bool hasWidget() const;
@@ -286,16 +285,18 @@ public:
 protected:
     void setModel(Model *model);
     void removeModel();
-    static WidgetInfo createWidgetInfo(QWidget *widget = nullptr,
-                                       const QString &uniqueId = QString(),
-                                       WidgetInfo::PlacementHint placementHint = WidgetInfo::NoPane,
-                                       int placementPriority = 0,
-                                       const QString &tabName = QString(), DesignerWidgetFlags widgetFlags = DesignerWidgetFlags::DisableOnError);
-
-private: //functions
-    QList<ModelNode> toModelNodeList(const QList<Internal::InternalNodePointer> &nodeList) const;
+    static WidgetInfo createWidgetInfo(
+        QWidget *widget = nullptr,
+        const QString &uniqueId = QString(),
+        WidgetInfo::PlacementHint placementHint = WidgetInfo::NoPane,
+        int placementPriority = 0,
+        const QString &tabName = QString(),
+        const QString &feedbackDisplayName = QString(),
+        DesignerWidgetFlags widgetFlags = DesignerWidgetFlags::DisableOnError);
 
 private:
+    QList<ModelNode> toModelNodeList(const QList<Internal::InternalNodePointer> &nodeList) const;
+
     QPointer<Model> m_model;
     ExternalDependenciesInterface &m_externalDependencies;
     bool m_enabled = true;
@@ -304,5 +305,4 @@ private:
 
 QMLDESIGNERCORE_EXPORT QList<Internal::InternalNodePointer> toInternalNodeList(const QList<ModelNode> &nodeList);
 QMLDESIGNERCORE_EXPORT QList<ModelNode> toModelNodeList(const QList<Internal::InternalNodePointer> &nodeList, AbstractView *view);
-
-}
+} // namespace QmlDesigner
